#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <allegro.h>
#include "052b.h"
#include "052bmix.h"

int end_game;			/* flag we'll set to end the game */

BITMAP *dblbuffer;

#define MAX_SHAPES      40

SHAPE_T *moving_shape[MAX_SHAPES];

int main(void)
{
    allegro_init();		/* initialise the Allegro library */
    init();			/* initialise the game */

    end_game = 0;		/* reset flag */

    do {			/* loop */
	input();		/* get input */
	process();		/* process it */
	output();		/* give output */
    } while (end_game == 0);	/* until the flag is set */

    shutdown();			/* shut down anything that needs it */
    allegro_exit();		/* just for luck */
    return 0;			/* tell the OS that all went well */
}
END_OF_MAIN()

void init(void)
{
    int f, what_type;

    srand(time(NULL));

    install_keyboard();

    if (set_gfx_mode(GFX_AUTODETECT, 320, 200, 0, 0) < 0) {
	allegro_message("Couldn't set gfx mode: %s\n", allegro_error);
	exit(1);
    }
    dblbuffer = create_bitmap(SCREEN_W, SCREEN_H);
    if (dblbuffer == NULL) {
	allegro_exit();
	allegro_message("Sorry, not enough memory.");
	exit(2);
    }
    clear(dblbuffer);

    for (f = 0; f < MAX_SHAPES; f++) {
	// Ok, since we have to possible shape types, we first have to chose one
	if (rand() % 2 == 0)
	    what_type = CIRCLE;
	else
	    what_type = SQUARE;

	moving_shape[f] = shape_init(what_type,
				     rand() % (SCREEN_W / 2),		// x
				      rand() % (SCREEN_H / 2),	// y
				      (rand() % 25) + 5,	// radius
				      (rand() % 25) + 5,	// width
				      (rand() % 25) + 5,	// height
				      (rand() % 15) + 1,	// color
				      (rand() % 7) - 3,
				     (rand() % 7) - 3);
	if (moving_shape[f] == NULL) {
	    allegro_exit();
	    allegro_message("Sorry, not enough memory.");
	    exit(3);
	}
	shape_draw(moving_shape[f], dblbuffer);
    }

    show_double_buffer();
}

void input(void)
{
    if (keypressed())
	end_game++;
}

void process(void)
{
    int f;

    for (f = 0; f < MAX_SHAPES; f++)
	shape_update(moving_shape[f]);
}

void output(void)
{
    int f;
    /* Notice the following, we have to separate shape_erase from the other
     * loop, or we could erase shapes while others were drawing themselves.
     */
    for (f = 0; f < MAX_SHAPES; f++)
	shape_erase(moving_shape[f], dblbuffer);

    for (f = 0; f < MAX_SHAPES; f++)
	shape_draw(moving_shape[f], dblbuffer);

    show_double_buffer();
}

void shutdown(void)
{
    int f;

    for (f = 0; f < MAX_SHAPES; f++)
	shape_destroy(moving_shape[f]);

    destroy_bitmap(dblbuffer);
}

void show_double_buffer(void)
{
    vsync();
    blit(dblbuffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
}
