#include <stdlib.h>
#include <allegro.h>
#include "052bmix.h"

SHAPE_T *shape_init(int new_type, int new_x, int new_y, int new_radius,
		    int new_width, int new_height, int new_col,
		    int new_xs, int new_ys)
{
    SHAPE_T *temp;

    // Allocate memory for the circle. If unsuccessful, return NULL.
    temp = malloc(sizeof(SHAPE_T));

    if (!temp)
	return NULL;

    temp->type = new_type;
    temp->x = new_x;
    temp->y = new_y;
    temp->r = new_radius;
    temp->w = new_width;
    temp->h = new_height;
    temp->col = new_col;
    temp->xs = new_xs;
    temp->ys = new_ys;
    temp->dx = temp->dy = 0;

    return temp;
}

void shape_destroy(SHAPE_T * shape)
{
    if (shape)
	free(shape);
}

void shape_draw(SHAPE_T * shape, BITMAP * bmp)
{
    switch (shape->type) {
    case CIRCLE:
	circlefill(bmp, shape->x, shape->y, shape->r, shape->col);
	shape->dx = shape->x;
	shape->dy = shape->y;
	break;

    case SQUARE:
	rectfill(bmp, shape->x, shape->y,
		 shape->x + shape->w, shape->y + shape->h, shape->col);
	shape->dx = shape->x;
	shape->dy = shape->y;
	break;
    }
}

void shape_update(SHAPE_T * shape)
{
    // First we update the position of the shape...
    shape->x += shape->xs;
    shape->y += shape->ys;

    // Now we see if it reached the maximum frame border

    switch (shape->type) {
    case CIRCLE:
	if (shape->x > SCREEN_W - 5)
	    shape->xs *= -1;

	if (shape->y > SCREEN_H - 5)
	    shape->ys *= -1;

	if (shape->x < 5)
	    shape->xs *= -1;

	if (shape->y < 5)
	    shape->ys *= -1;
	break;
    case SQUARE:
	if (shape->x + (shape->w / 2) > SCREEN_W - 5)
	    shape->xs *= -1;

	if (shape->y + (shape->h / 2) > SCREEN_H - 5)
	    shape->ys *= -1;

	if (shape->x + (shape->w / 2) < 5)
	    shape->xs *= -1;

	if (shape->y + (shape->h / 2) < 5)
	    shape->ys *= -1;
	break;
    }
}

void shape_erase(SHAPE_T * shape, BITMAP * bmp)
{
    switch (shape->type) {
    case CIRCLE:
	circlefill(bmp, shape->dx, shape->dy, shape->r, 0);
	break;
    case SQUARE:
	rectfill(bmp, shape->dx, shape->dy,
		 shape->dx + shape->w, shape->dy + shape->h, 0);
	break;
    }
}
