#include <stdio.h>
#include <allegro.h>
#include "0622.h"
#include "0622c.h"

int end_game;			/* flag we'll set to end the game */
int pressed_fire_button;	// This wii help with the color feature

BITMAP *dblbuffer;

CIRCLE_T *moving_circle;

int main(void)
{
    allegro_init();		/* initialise the Allegro library */
    init();			/* initialise the game */

    end_game = 0;		/* reset flag */

    do {			/* loop */
	input();		/* get input */
	process();		/* process it */
	output();		/* give output */
    } while (end_game == 0);	/* until the flag is set */

    shutdown(); 		/* shut down anything that needs it */
    allegro_exit();		/* just for luck */
    return 0;			/* tell the OS that all went well */
}
END_OF_MAIN()

void init(void)
{
    if (install_joystick(JOY_TYPE_AUTODETECT) != 0) {
	allegro_message("\nSorry, couldn't detect a joystick\n");
	exit(1);
    };
    if (num_joysticks == 0) {
	allegro_message("\nSorry, couldn't detect a joystick\n");
	exit(2);
    };
    install_keyboard();

    if (set_gfx_mode(GFX_AUTODETECT, 320, 200, 0, 0) < 0) {
	allegro_message("Couldn't set gfx mode: %s\n", allegro_error);
	exit(1);
    }
    dblbuffer = create_bitmap(SCREEN_W, SCREEN_H);
    if (dblbuffer == NULL) {
	allegro_exit();
	allegro_message("Sorry, not enough memory");
	exit(2);
    }
    clear(dblbuffer);

    moving_circle = circle_init(SCREEN_W / 2, SCREEN_H / 2, 50, 3, 0, 0);
    if (moving_circle == NULL) {
	allegro_exit();
	allegro_message("Sorry, not enough memory");
	exit(3);
    }
    pressed_fire_button = 0;

    circle_draw(moving_circle, dblbuffer);
    show_double_buffer();
}

void input(void)
{
    if (key[KEY_ESC])
	end_game++;

    poll_joystick();

    /* Simple trick. Since we want to change the color only when the joystick
     * button state changes from not pressed to pressed, we store in 
     * pressed_fire_button the actual value of the button, and compare it in 
     * next loops. If it's different, then we change color and update our 
     * variable. We also make sure that the new color will be different from 
     * the old one using the global variable as a temporary holder of the 
     * color and comparing it with the new rand value.
     */

    if (joy_b1 && !pressed_fire_button) {
	pressed_fire_button = moving_circle->col;
	while (pressed_fire_button == moving_circle->col)
	    moving_circle->col = rand() % 14;
    }
    pressed_fire_button = joy_b1;

    if (joy_left)
	moving_circle->xs--;	// if the user is pressing left

    if (joy_right)
	moving_circle->xs++;	// arrow, go left same for right

    if (joy_up)
	moving_circle->ys--;	// same for up; note that y _decreases_

    if (joy_down)
	moving_circle->ys++;	// ... and for down y _increases_

}

void process(void)
{
    circle_update(moving_circle);
}

void output(void)
{
    circle_erase(moving_circle, dblbuffer);
    circle_draw(moving_circle, dblbuffer);

    show_double_buffer();
}

void shutdown(void)
{
    circle_destroy(moving_circle);
    destroy_bitmap(dblbuffer);
}

void show_double_buffer(void)
{
    vsync();
    blit(dblbuffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
}
