#include <stdio.h>
#include <allegro.h>
#include "circle.h"
#include "main.h"
#include "input.h"

#define MAX_MOVE_SPEED	     5

int end_game;				    /* flag we'll set to end the game */

BITMAP *dblbuffer;
CIRCLE_T *moving_circle[3];		    /* We want three circles this time. */
int pressed_fire_button[3]={0,0,0};

int main (void)
{
    allegro_init();			    /* initialise the Allegro library */
    init();				    /* initialise the game */

    end_game = 0;			    /* reset flag */

    do {				    /* loop */
	input();			    /* get input */
	process();			    /* process it */
	output();			    /* give output */
    } while (end_game == 0);		    /* until the flag is set */

    shutdown(); 			    /* shut down anything that needs it */
    allegro_exit();			    /* just for luck */
    return 0;				    /* tell the OS that all went well */
}
END_OF_MAIN()

void init(void)
{
    install_timer();

    if( set_gfx_mode(GFX_AUTODETECT,640,480,0,0) < 0){
	allegro_message("Couldn't set gfx mode: %s\n", allegro_error);
	exit(1);
	}

    dblbuffer=create_bitmap(SCREEN_W,SCREEN_H);
    if(dblbuffer == NULL) {
	allegro_exit();
	allegro_message("Sorry, not enough memory");
	exit(2);
    }
    clear(dblbuffer);

    moving_circle[0] = circle_init(SCREEN_W/2,SCREEN_H/2,50,3,0,0);
    moving_circle[1] = circle_init(SCREEN_W/2,(SCREEN_H/2)-50,50,4,0,0);
    moving_circle[2] = circle_init(SCREEN_W/2,(SCREEN_H/2)+50,50,5,0,0);
    if(!moving_circle[0] || !moving_circle[1] || !moving_circle[2]) {
	allegro_exit();
	allegro_message("Sorry, not enough memory");
	exit(3);
    }

    if ( input_init() != 0) {
	allegro_exit();
	allegro_message("Sorry, couldn't initialise input.\n");
	exit(4);
    }

    circle_draw(moving_circle[0],dblbuffer);
    circle_draw(moving_circle[0],dblbuffer);
    circle_draw(moving_circle[0],dblbuffer);
    show_double_buffer();
}

void process(void)
{
    int f;

    for (f=0; f < 3; f++) {

	// Check that the input device is "on"
	if (all_input[f] == NULL) continue;

	// First we update the position of the circle...
	moving_circle[f]->x += all_input[f]->dx / (128 / MAX_MOVE_SPEED);
	moving_circle[f]->y += all_input[f]->dy / (128 / MAX_MOVE_SPEED);
    
	if (all_input[f]->fire1 != pressed_fire_button[f]) {
	    pressed_fire_button[f] = moving_circle[f]->col;
	    while (pressed_fire_button[f] == moving_circle[f]->col)
		moving_circle[f]->col = rand()%14;
    
	    pressed_fire_button[f] = all_input[f]->fire1;
	}
     
	circle_update(moving_circle[f]);
    }
}

void output(void)
{
    int f;

    for (f=0; f<3; f++)
	if (all_input[f] != NULL)
	    circle_erase(moving_circle[f],dblbuffer);
    for (f=0; f<3; f++)
	if (all_input[f] != NULL)
	    circle_draw(moving_circle[f],dblbuffer);

    text_mode(-1);
    textout_centre(dblbuffer,font,"Press ESC to see menu",SCREEN_W>>1,
				 SCREEN_H- text_height(font),15);

    show_double_buffer();
}

void shutdown(void)
{
    input_shutdown ();
    circle_destroy(moving_circle[0]);
    circle_destroy(moving_circle[1]);
    circle_destroy(moving_circle[2]);
    destroy_bitmap(dblbuffer);
}

void show_double_buffer(void)
{
    vsync();
    blit(dblbuffer,screen,0,0,0,0,SCREEN_W,SCREEN_H);
}
